//
//  FilmAffinity.m

#import "FilmAffinity.h"
#import "Constants.h"
#import "CSRegex.h"

@implementation FilmAffinity

- (void)searchFor:(NSDictionary *)searchDict sender:(id)sender{
	delegate = sender;
	[resultsTitle removeAllObjects];
	[resultsURL removeAllObjects];
	searchDepth = 0;
	
	NSMutableString *searchString = [NSMutableString string];
	
	
	if ([searchDict objectForKey:@"keyword"]) {
		[searchString setString:[searchDict objectForKey:@"keyword"]];
	}
	else if ([searchDict objectForKey:@"url"] != nil) {
		NSString *url = [searchDict objectForKey:@"url"];
		if ([url rangeOfString:@"www.filmaffinity.com"].location != NSNotFound) {
			[resultsTitle addObject:url];
			[resultsURL addObject:url];	
			[delegate searchReturnedNumberOfResults:[resultsURL count] sender:self];
			return;
		}
	}	
	else if ([searchDict objectForKey:@"title"] != nil) {
		[searchString setString:[searchDict objectForKey:@"title"]];
	}
	else {
		[delegate searchReturnedNumberOfResults:0 sender:self];
		return;
	}
	
	//NSString *encodedString = (NSString *)CFURLCreateStringByAddingPercentEscapes(NULL, (CFStringRef)searchString, NULL, (CFStringRef)@"!*'();:@&=+$,/?%#[]", kCFStringEncodingUTF8 );
	
	NSString *encodedString = [searchString stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
	
	[searchedURL release];
	searchedURL = [[NSString stringWithFormat:@"http://www.filmaffinity.com/es/search.php?stext=%@&stype=all", encodedString] retain];
	
	//http://www.filmaffinity.com/en/search.php?stext=%5B%95REC%5D+&stype=title
	//http://www.filmaffinity.com/es/search.php?stext=%5B%E2%80%A2REC%5D&stype=all
	//NSLog(@"%@", searchedURL);
	
	[self downloadWebPage: searchedURL];
}


- (void)downloadComplete:(NSString *)HTMLSource {
	// Search the page and create a list of result titles and URLs for the details
	// In this sample we pick all the links that start with "http"
	
	NSScanner *tempScanner = [NSScanner scannerWithString:HTMLSource];
	
	[tempScanner scanUpToString:@"Resultados por " intoString:nil];
	[tempScanner scanUpToString:@"</div>" intoString:nil];
	
	while (![tempScanner isAtEnd]) {
		NSString *theURL = nil;
		NSString *theTitle = nil;
		[tempScanner scanUpToString:@"<b><a href=" intoString: nil];
		[tempScanner scanUpToString:@"\"" intoString: nil];
		[tempScanner scanUpToString:@"\">" intoString:&theURL];
		[tempScanner scanUpToString:@"</a>" intoString:&theTitle];
		
		if (theURL && theTitle && [theURL hasPrefix:@"\""] && [theTitle hasPrefix:@"\">"]) {
			theURL = [NSString stringWithFormat:@"http://www.filmaffinity.com%@", [theURL substringFromIndex:1]];
			//make sure it's not already added
			if (![resultsURL containsObject:theURL]) {
				theTitle = [[theTitle substringFromIndex:2] stringByCleaningHTML];
				[resultsTitle addObject:theTitle];
				[resultsURL addObject:theURL];
			}
			
		}
	}
	
	if ([resultsTitle count] == 0){
		//single result
		//if ([HTMLSource rangeOfString:@"TITULO ORIGINAL" options:NSCaseInsensitiveSearch].location !+= NSNotFound) {
		NSString *theTitle = nil;
		[tempScanner setScanLocation:0];
		[tempScanner scanUpToString:@"TULO ORIGINAL<" intoString:nil];
		
		[tempScanner scanUpToString:@"<b>" intoString: nil];
		[tempScanner scanUpToString:@"</" intoString:&theTitle];
		if (theTitle && [theTitle hasPrefix:@"<b>"]) {
			theTitle = [[theTitle substringFromIndex:3] stringByCleaningHTML];
			[resultsTitle addObject:theTitle];
			[resultsURL addObject:[@"HTMLSource:" stringByAppendingString:HTMLSource]];
		}
		//}
		
	}
	// <a href="search.php?stype=director&stext=howard+Hawks&from=5">
	NSString *possibeMoreResults = [HTMLSource stringBetween:@"href=\"search.php?stype=" and:@"\""];
	if (possibeMoreResults && searchDepth < 3) {
		possibeMoreResults = [@"http://www.filmaffinity.com/es/search.php?stype=" stringByAppendingString:possibeMoreResults];
		[self downloadWebPage: possibeMoreResults];
		searchDepth++;
	}
	else
		[delegate searchReturnedNumberOfResults:[resultsURL count] sender:self];	
}


- (NSDictionary *)resultNumber:(int)number {
	//Get the details for one of the results and return it as a dictionary
	
	NSMutableDictionary *returnData = [NSMutableDictionary dictionaryWithCapacity:10];
	
	if (number >= [resultsURL count]) {
		NSLog(@"Error: FilmAffinity requested out of range: %d of %d", number, [resultsURL count]);
		[returnData setValue:@"No data found on FilmAffinity" forKey:@"title"];
		return returnData;	
	}
	
	NSString *httpURL = [resultsURL objectAtIndex:number];
	
	NSString *results;
	if ([httpURL hasPrefix:@"HTMLSource:"]) {
		results = httpURL;
		httpURL = searchedURL;
	}
	else {
		NSURLRequest *theRequest=[NSURLRequest requestWithURL:[NSURL URLWithString:httpURL] cachePolicy:NSURLRequestUseProtocolCachePolicy timeoutInterval:15.0];
		
		results = [[[NSString alloc] initWithData:[NSURLConnection sendSynchronousRequest:theRequest returningResponse:nil error:nil] encoding:NSISOLatin1StringEncoding] autorelease];
	}
	
	if (results == nil)
		return nil;
	
	if (httpURL)
		[returnData setValue:httpURL forKey:@"url"];
	
	//Release Date fill in Release publication
	
	NSString *title = [resultsTitle objectAtIndex:number];
	if (title)
		[returnData setValue:title forKey:@"title"];
	
	//NSArray* rg = [results substringsCapturedByPattern:@"<b>DIRECTOR</b></td>[^<]*<td ><a href=\"[^\"]+\">([^<]+)"];
	
	NSString *director = [results stringBetween:@"<b>DIRECTOR</b></td>" and:@"</td>"];
	director = [director stringByCleaningHTML];
	if (director) {
		NSMutableString *directorCleanSpace = [[director mutableCopy] autorelease];
		[directorCleanSpace replaceOccurrencesOfString:@"  " withString:@" " options:NSLiteralSearch range:NSMakeRange(0, [directorCleanSpace length])];
		[returnData setValue:directorCleanSpace forKey:MKKeyDVDDirector];
	}
	
	NSArray *rg = [results substringsCapturedByPattern:@"TULO ORIGINAL</b></td>[^<]*<td ><b>([^<]+)"];
	if ( [rg count] > 0) {
		[returnData setValue:[rg objectAtIndex:1] forKey:@"originalTitle"];
	}
	
	rg = [results substringsCapturedByPattern:@"<b>AÑO</b></td>[^<]*<td>[^<]*<table width=\"100%\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\"><tr>[^<]*<td >([^<]+)"];
	if ( [rg count] > 0) {
		[returnData setValue:[rg objectAtIndex:1] forKey:MKKeyDVDTheatrical];
	}
	
	rg = [results substringsCapturedByPattern:@"<b>PRODUCTORA</b></td>[^<]*<td  >([^<]+)"];
    if ( [rg count] > 0) {
		[returnData setValue:[rg objectAtIndex:1] forKey:MKKeyDVDProducer];
	}
	
	NSString *country = [results stringBetween:@"PAÍS</b></td>" and:@"</td>"];
	country = [country stringBetween:@"title=\"" and:@"\""];
	[returnData setValue:country forKey:MKKeyDVDCountry];
	
	/*rg = [results substringsCapturedByPattern:@"<b>PAÍS</b></td>[^<]*<td ><img src=\"[^\"]+\" title=\"([^\"]+)"];
	 if ( [rg count] > 0) {
	 [returnData setValue:[rg objectAtIndex:1] forKey:@"country"];
	 }
	 */
	
	rg = [results substringsCapturedByPattern:@"<b>GUIÓN</b></td>[^<]*<td >([^<]+)"];
    if ( [rg count] > 0) {
		[returnData setValue:[[rg objectAtIndex:1] stringByCleaningHTML] forKey:MKKeyDVDWriter];
	}
	
	rg = [results substringsCapturedByPattern:@"center\" style=\"color:#990000;[^>]+>([^<]+)"];
	if ( [rg count] > 0) {
		[returnData setValue:[rg objectAtIndex:1] forKey:MKKeyDVDViewerRating];
	}
	
	NSString *pre_raw_starrings = [results stringBetween:@"<b>REPARTO</b></td>" and:@"</td>"];
	if (pre_raw_starrings) {
		NSMutableString *starrings = [NSMutableString string];
		NSArray *raw_starrings = [pre_raw_starrings componentsSeparatedByString:@", "];
		NSString *element;
		int c = [raw_starrings count];
		int i;
		for (i=0; i<c; i++) {
			element = [raw_starrings objectAtIndex:i];
			NSString *creditName = [[element stringBetween:@"\">" and:@"<"] stringByRemovingLeadingAndTrailingSpaces];
			if (creditName && ![creditName isEqualToString:@"Animation"]) {
				if (i == 0) {
					[starrings setString:creditName];
				}
				else {
					[starrings appendFormat:@", %@", creditName];
				}
			}
		}
		if ([starrings length])
			[returnData setValue:starrings forKey:MKKeyDVDStarring];
	}
	
	NSString *officialWebPage = [results stringBetween:@"WEB OFICIAL</b></td>" and:@"</td>"];
	officialWebPage = [officialWebPage stringByCleaningHTML];
	if (officialWebPage) {
		NSMutableArray *linksArray = [NSMutableArray array];
		[linksArray addObject:[NSDictionary dictionaryWithObjectsAndKeys:officialWebPage, @"url", @"Página Oficial", @"name", @"Página oficial", @"type", nil]];
		[returnData setValue:linksArray forKey:@"links"];
	}
	
	rg = [results substringsCapturedByPattern:@"<b>DURACIÓN</b></td>[^<]*<td>[^<]*<table[^>]+><tr>[^<]*<td>([^<]+)"];
	if ( [rg count] > 0) {
		NSString *duration = [rg objectAtIndex:1];
		if ([duration intValue])
			[returnData setValue:duration forKey:MKKeyDVDDuration];
	}
	
	
	rg = [results substringsCapturedByPattern:@"href=\"(http://pics2?.filmaffinity.com/[^\"]+)"];
	
	if ( [rg count] > 0) {
		NSString *imageLocation = [rg objectAtIndex:1];
		[returnData setValue:imageLocation forKey:MKKeyEntryImageLocation];
	}
	
	NSString *rating = [results stringBetween:@"style=\"color:#990000; font-size:22px;" and:@"</td>"];
	rating = [rating stringBetween:@">" and:@"<"];
	if (rating)
		[returnData setValue:rating forKey:MKKeyDVDViewerRating];
	
	NSString *genre = [results stringBetween:@"<b>GÉNERO Y CRÍTICA" and:@"</tr>"];
	genre = [genre stringBetween:@"</td>" and:@"</tr>"];
	if (genre) {
		genre = [genre stringByCleaningHTML];
		
		NSRange sinopsis = [genre rangeOfString:@" / SINOPSIS: "];
		if (sinopsis.location != NSNotFound) {
			
			NSString *summary = [genre substringFromIndex:sinopsis.location + sinopsis.length];
			genre = [genre substringToIndex:sinopsis.location];
			
			unsigned int slashLocation = [genre rangeOfString:@" / "].location; 
			if (slashLocation != NSNotFound) { //Premios come before
				NSString *premios = [genre substringToIndex:slashLocation];
				genre = [genre substringFromIndex:slashLocation + 3];
				[returnData setValue:premios forKey:MKKeyEntryAwards];
			}
			
			//if ([genre rangeOfString:@"."].location != NSNotFound) {
			NSMutableString *spreadGenre = [NSMutableString stringWithString:genre]; 
			[spreadGenre replaceOccurrencesOfString:@"." withString:@"," options:NSLiteralSearch range:NSMakeRange(0, [spreadGenre length])];
			[spreadGenre replaceOccurrencesOfString:@" | " withString:@", " options:NSLiteralSearch range:NSMakeRange(0, [spreadGenre length])];
			[spreadGenre replaceOccurrencesOfString:@" | " withString:@", " options:NSLiteralSearch range:NSMakeRange(0, [spreadGenre length])];
			[spreadGenre replaceOccurrencesOfString:@"\t" withString:@" " options:NSLiteralSearch range:NSMakeRange(0, [spreadGenre length])];
			[spreadGenre replaceOccurrencesOfString:@"  " withString:@" " options:NSLiteralSearch range:NSMakeRange(0, [spreadGenre length])];
			[spreadGenre replaceOccurrencesOfString:@"  " withString:@" " options:NSLiteralSearch range:NSMakeRange(0, [spreadGenre length])];
			
			
			genre = spreadGenre;
			[returnData setValue:genre forKey:MKKeyEntryGenre];
			
			[returnData setValue:summary forKey:MKKeyEntrySummary];
		}
		
	}
	
	
	genre = [results stringBetween:@"<b>GÉNERO</b>" and:@"</tr>"];
	if (genre) {
		genre = [genre stringByCleaningHTML];
		
		//if ([genre rangeOfString:@"."].location != NSNotFound) {
		NSMutableString *spreadGenre = [NSMutableString stringWithString:genre]; 
		[spreadGenre replaceOccurrencesOfString:@"." withString:@"," options:NSLiteralSearch range:NSMakeRange(0, [spreadGenre length])];
		[spreadGenre replaceOccurrencesOfString:@" | " withString:@", " options:NSLiteralSearch range:NSMakeRange(0, [spreadGenre length])];
		[spreadGenre replaceOccurrencesOfString:@" | " withString:@", " options:NSLiteralSearch range:NSMakeRange(0, [spreadGenre length])];
		[spreadGenre replaceOccurrencesOfString:@"\t" withString:@" " options:NSLiteralSearch range:NSMakeRange(0, [spreadGenre length])];
		[spreadGenre replaceOccurrencesOfString:@"  " withString:@" " options:NSLiteralSearch range:NSMakeRange(0, [spreadGenre length])];
		[spreadGenre replaceOccurrencesOfString:@"  " withString:@" " options:NSLiteralSearch range:NSMakeRange(0, [spreadGenre length])];
		
		
		genre = spreadGenre;
		//}
		
		[returnData setValue:genre forKey:MKKeyEntryGenre];
	}
	
	
	
	
	NSString *awards = [results stringBetween:@"<b>PREMIOS</b>" and:@"</tr>"];
	if (awards) {
		awards = [awards stringByCleaningHTML];
		
		if (awards) {
			NSMutableString *spreadGenre = [NSMutableString stringWithString:awards]; 
			[spreadGenre replaceOccurrencesOfString:@"\r" withString:@"; " options:NSLiteralSearch range:NSMakeRange(0, [spreadGenre length])];
			[spreadGenre replaceOccurrencesOfString:@"\n" withString:@"" options:NSLiteralSearch range:NSMakeRange(0, [spreadGenre length])];
			
			awards = spreadGenre;
		}
		
		
		[returnData setValue:awards forKey:MKKeyEntryAwards];
	}
	
	
	
	
	NSString *raw_summary = [results stringBetween:@"<b>SINOPSIS" and:@"</tr>"];
	if (raw_summary) {
		
		NSMutableString *summary = [NSMutableString stringWithString:raw_summary];
		[summary replaceOccurrencesOfString:@"\t" withString:@"" options:NSCaseInsensitiveSearch range:NSMakeRange(0,[summary length])];
		[summary replaceOccurrencesOfString:@"\r" withString:@"" options:NSCaseInsensitiveSearch range:NSMakeRange(0,[summary length])];
		[summary replaceOccurrencesOfString:@"<br />" withString:@"\n" options:NSCaseInsensitiveSearch range:NSMakeRange(0,[summary length])];
		[summary replaceOccurrencesOfString:@"<br>" withString:@"\n" options:NSCaseInsensitiveSearch range:NSMakeRange(0,[summary length])];
		[summary replaceOccurrencesOfString:@"&quot;" withString:@"\"" options:NSCaseInsensitiveSearch range:NSMakeRange(0,[summary length])];
		[summary replaceOccurrencesOfString:@"<b>" withString:@"" options:NSCaseInsensitiveSearch range:NSMakeRange(0,[summary length])];
		[summary replaceOccurrencesOfString:@"</b>" withString:@"" options:NSCaseInsensitiveSearch range:NSMakeRange(0,[summary length])];
		
		if (summary){
			[returnData setValue:[summary stringByCleaningHTML] forKey:MKKeyEntrySummary];
		}
	}
	
	return returnData;
}


#pragma mark -
#pragma mark Getter

- (NSArray *)resultsTitles {
	return resultsTitle;
}


#pragma mark Init
+ (id)plugin;
{ 
    return [[[self alloc] init] autorelease];
}


- (id)init {
	if (self = [super init]) {
		resultsTitle = [[NSMutableArray array] retain];
		resultsURL = [[NSMutableArray array] retain];
	}
	return self;
}


- (void)dealloc {
	[searchedURL release];
	[resultsTitle release];
	[resultsURL release];
	[super dealloc];
}




#pragma mark Connection Delegation
- (void)downloadWebPage:(NSString *)aWebPage {
	//Create request for http query
	NSURLRequest *httpRequest = [NSURLRequest requestWithURL:[NSURL URLWithString:aWebPage] cachePolicy:NSURLRequestUseProtocolCachePolicy timeoutInterval:30.0];
	
	theConnection = [[NSURLConnection alloc] initWithRequest:httpRequest delegate:self];
	
	if (theConnection) {
		receivedData = [[NSMutableData data] retain];
	}
	else {
		[delegate searchReturnedNumberOfResults:0 sender:self];
	}	
}


- (void)downloadWebPage:(NSString *)aWebPage withPost:(NSString *)body encoding:(int)anEncoding {
	//Create request for http query
	NSMutableURLRequest *postRequest = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:aWebPage] cachePolicy:NSURLRequestUseProtocolCachePolicy timeoutInterval:30.0];
	[postRequest setHTTPMethod: @"POST"];
	
	[postRequest setHTTPBody:[body dataUsingEncoding:anEncoding]];
	
	theConnection = [[NSURLConnection alloc] initWithRequest:postRequest delegate:self];
	
	if (theConnection) {
		receivedData = [[NSMutableData data] retain];
	}
	else {
		[delegate searchReturnedNumberOfResults:0 sender:self];
	}	
	
}



/* If there are any redirects this notification is sent and we reset the data to 0  */
- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response
{
	//redirects and such
    [receivedData setLength:0];
}



- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
    // append the new data to the receivedData
    [receivedData appendData:data];
}


- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
    // release the connection, and the data object
    [connection release];
    [receivedData release];
	receivedData = nil;
	[delegate searchReturnedNumberOfResults:0 sender:self];
}


-(NSURLRequest *)connection:(NSURLConnection *)connection willSendRequest:(NSURLRequest *)request redirectResponse:(NSURLResponse *)redirectResponse {
	[searchedURL release];
	searchedURL = [[[redirectResponse URL] absoluteString] retain];
	return request;
}


- (void)connectionDidFinishLoading:(NSURLConnection *)connection
{
	//Turn recievedData into a  string try NSUTF encoding if that fails use Latin
	NSString *HTMLSource = [[[NSString alloc] initWithData:receivedData encoding:NSUTF8StringEncoding] autorelease];	
	if (HTMLSource == nil)
		HTMLSource = [[[NSString alloc] initWithData:receivedData encoding:NSISOLatin1StringEncoding] autorelease];	

    // release the connection, and the data object
    [connection release];
    [receivedData release];
	receivedData = nil;
	
	//All encodings failed return no results otherwise parse
	if (HTMLSource == nil)
		[delegate searchReturnedNumberOfResults:0 sender:self];
	else
		[self downloadComplete:HTMLSource];
	
}

@end
